#########################################################################
#########################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# R program for fitting Generalized Linear Models to analyse 
# time series of proportion of A. salmon 1+ Smolts in French rivers
# (logistic and probit regressions)
# Chapter 8 - section 8.1

##############################################################################
##############################################################################



rm(list = ls())

library(rbugs)
library(boot)
library("BRugs")
library(coda)
library("mnormt") 


# Data used only for graphical representation
data_GLMpSm1 <- read.table(file = "data_GLMpSm1_table.txt", header = T, sep = "")

# Select data for Year >= 1973 (Year = 1 = 1964 ; Year = 40 = 2003) 
data_GLMpSm1 <- subset(data_GLMpSm1, subset=c(Year>=10))




# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                           I. Run BUGS programs
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------


# ------------------------------------------------------------------------------------
# MCMC config.
# Be careful to the number of iterations, the model runs slowly
# ------------------------------------------------------------------------------------

NburnIn <- 10000 		# Burn in phase
Niter <- 5000		# Inferences
thin <- 10
n.chains <- 3


# ------------------------------------------------------------------------------------
# Modele M0 (pSm1 ~ Year + Region + SeaAge) 
# ------------------------------------------------------------------------------------

model <- "model_GLMpSm1_Zellner.txt"
modelCheck(model)  

model.version <- "M0"
data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")

modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(NburnIn, thin = 1) 

# Inferences

samplesSet("theta") 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame
theta <- as.data.frame(as.matrix(buildMCMC("theta")))

# Posterior pdf of parameters
# windows()
# k <- source(data)$value$k
# boxplot(theta, xaxt = "n", yaxt = "n", ylim = c(-1,1))
# axis(side =1, at=1:k, labels = 1:k)

file_to_save <- paste("theta_",model.version,".txt",sep = "")
write.table(theta, file = file_to_save, 
            append = FALSE, row.names = TRUE, col.names = TRUE, quote = FALSE)




# ------------------------------------------------------------------------------------
# Modele M1 (pSm1 ~ Year + SeaAge) 
# ------------------------------------------------------------------------------------

modelCheck(model)  

model.version <- "M1"
data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")

modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

samplesSet("theta") 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame

theta <- as.data.frame(as.matrix(buildMCMC("theta")))

# Posterior pdf of parameters
# windows()
# k <- source(data)$value$k
# boxplot(theta, xaxt = "n", yaxt = "n", ylim = c(-1,1))
# axis(side =1, at=1:k, labels = 1:k)

file_to_save <- paste("theta_",model.version,".txt",sep = "")
write.table(theta, file = file_to_save, 
            append = FALSE, row.names = TRUE, col.names = TRUE, quote = FALSE)





# ------------------------------------------------------------------------------------
# Modele M2 (pSm1 ~ Year + Region) 
# ------------------------------------------------------------------------------------

modelCheck(model)  

model.version <- "M2"
data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")

modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

samplesSet("theta") 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame

theta <- as.data.frame(as.matrix(buildMCMC("theta")))

# posterior pdf of parameters 
# windows()
# k <- source(data)$value$k
# boxplot(theta, xaxt = "n", yaxt = "n", ylim = c(-1,1))
# axis(side =1, at=1:k, labels = 1:k)

file_to_save <- paste("theta_",model.version,".txt",sep = "")
write.table(theta, file = file_to_save, 
            append = FALSE, row.names = TRUE, col.names = TRUE, quote = FALSE)



                                                                  
# ------------------------------------------------------------------------------------
# Modele M3 (pSm1 ~ Year*SeaAge + Region) 
# ------------------------------------------------------------------------------------


modelCheck(model)  

model.version <- "M3"
data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")

modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

samplesSet("theta") 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame

theta <- as.data.frame(as.matrix(buildMCMC("theta")))

# posterior pdf of parameters
# windows()
# k <- source(data)$value$k
# boxplot(theta, xaxt = "n", yaxt = "n", ylim = c(-1,1))
# axis(side =1, at=1:k, labels = 1:k)

file_to_save <- paste("theta_",model.version,".txt",sep = "")
write.table(theta, file = file_to_save, 
            append = FALSE, row.names = TRUE, col.names = TRUE, quote = FALSE)




# ------------------------------------------------------------------------------------
# Modele M4 (pSm1 ~ Year*SeaAge + Region) 
# ------------------------------------------------------------------------------------


modelCheck(model)  

model.version <- "M4"
data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")

modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

samplesSet("theta") 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame

theta <- as.data.frame(as.matrix(buildMCMC("theta")))

# posterior pdf of parameters
# windows()
# k <- source(data)$value$k
# boxplot(theta, xaxt = "n", yaxt = "n", ylim = c(-1,1))
# axis(side =1, at=1:k, labels = 1:k)

file_to_save <- paste("theta_",model.version,".txt",sep = "")
write.table(theta, file = file_to_save, 
            append = FALSE, row.names = TRUE, col.names = TRUE, quote = FALSE)
       




# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                 II. Make graphics for 1 particular model
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------

model.version <- "M0"

n.years <- 31
years <- 1973:2003

data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
X <- source(data)$value$X
k <- dim(X)[2]
n <- source(data)$value$n

X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

# Get results
file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")
G <- dim(theta)[1]
mu<-theta[,1]
alpha<-theta[,2]
beta<-theta[,5]
gamma<-theta[,3]
delta<-theta[,4]

theta <- cbind(theta,-theta[,3]-theta[,4])

pSm1_predAge1<-matrix(NA,nr=G,nc=n.years )
pSm1_predAge2<-matrix(NA,nr=G,nc=n.years )
pSm1_predReg1<-matrix(NA,nr=G,nc=n.years )
pSm1_predReg2<-matrix(NA,nr=G,nc=n.years )
pSm1_predReg3<-matrix(NA,nr=G,nc=n.years )

# First year is t=10 in the data file

for (t in 1:n.years){
pSm1_predAge1[,t]<-mu + alpha*(t+9) + beta
pSm1_predAge2[,t]<-mu + alpha*(t+9) - beta
pSm1_predReg1[,t]<-mu + alpha*(t+9) + gamma
pSm1_predReg2[,t]<-mu + alpha*(t+9) + delta
pSm1_predReg3[,t]<-mu + alpha*(t+9) - gamma-delta
}
    

# -----------------------------------------------------
# Per sea-age class
# -----------------------------------------------------

# windows()

res <- 6
name_figure <- "SeaAgePredict.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), bty="n")

col <- c("black","white")
line <- c(1,2)
line.wd = c(1,1)
size.text <- 1.3
box.size = 0.3
size.labels = 1

pSm1_predAge1 <- as.data.frame(inv.logit(pSm1_predAge1))     
pSm1_predAge2 <- as.data.frame(inv.logit(pSm1_predAge2)) 

boxplot(	pSm1_predAge1,xlab="", ylab="", xaxt = "n", yaxt = "n",
		col=col[1], outline=F, names=paste(years), boxwex=box.size, ylim=c(0.3,1))
boxplot(	pSm1_predAge2, add=T, show.names=F, col=col[2],xaxt = "n", yaxt = "n",
		outline=F,names=rep("",n.years), boxwex=box.size, at = (1:n.years)+0.15)

empiric<-subset(data_GLMpSm1,subset=(SeaAge==1),select=-SeaAge)
estimageAge1<-tapply(empiric$y1/empiric$ytot ,empiric$Year,mean)

empiric<-subset(data_GLMpSm1,subset=(SeaAge==2),select=-SeaAge)
estimageAge2<-tapply(empiric$y1/empiric$ytot ,empiric$Year,mean)

lines(estimageAge1,col="black",lty=line[1], lwd = line.wd[1])
lines(estimageAge2,col="black",lty=line[2], lwd = line.wd[2])

axis(side =1, at=1:n.years, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)

mtext(text = "Years (cohort birth date)", side = 1, line = 4, las = 0, cex = size.text)
mtext(text = "Prop. Sm1+", side = 2, line = 3, las = 0, cex = size.text)

legend(25,0.45, title = "Sea-age class", c("1SW", "2SW"), fill = col, bty ="n") 

dev.off()


# -----------------------------------------------------
# Per Region
# -----------------------------------------------------

# windows()

res <- 6
name_figure <- "RegionPredict.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), bty="n")

col <- c("black","grey50","white")
size.text <- 1.3
box.size = 0.3
size.labels = 1

pSm1_predReg1 <-as.data.frame(inv.logit(pSm1_predReg1)) #effet de la region 1
pSm1_predReg2 <- as.data.frame(inv.logit(pSm1_predReg2))  #effet de la region 2
pSm1_predReg3 <- as.data.frame(inv.logit(pSm1_predReg3))    #effet de la region 3

boxplot(	pSm1_predReg1,xlab="",ylab="", xaxt = "n", yaxt = "n",
		col=col[1],outline=F,names=paste(years),boxwex=box.size,ylim = c(0.4,1))
boxplot(	pSm1_predReg2, add=T, show.names=F, xaxt = "n", yaxt = "n",
		col=col[2],outline=F,names=rep("",n.years), boxwex=box.size, at= (1:n.years)+0.15)
boxplot(	pSm1_predReg3, add=T, show.names=F, xaxt = "n", yaxt = "n",
		col=col[3],outline=F,names=rep("",n.years),boxwex=box.size,at= (1:n.years)+0.30)

axis(side =1, at=1:n.years, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)

mtext(text = "Years (cohort birth date)", side = 1, line = 4, las = 0, cex = size.text)
mtext(text = "Prop. Sm1+", side = 2, line = 3, las = 0, cex = size.text)

print.result <- function(x, ...)
{  
y<-(c(mean(x),var(x)^0.5,quantile(x,pr=c(0.05,0.25,0.5,0.75,0.95))))
print(floor(1000*y)/1000)
res<-y
}                             
apply(theta,2,print.result)

legend(26,0.55, c("A", "B", "C"), fill = col, bty ="n", title = "Regions") 

dev.off()






# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#                           III. Compute Bayes Factors
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------

memory.limit(size = 4000)
source("routine_LogJointLogit_Zellner.R")   

# ---------------------------------------------------------
 model.version <- "M0"
# ---------------------------------------------------------


data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
n <- length(source(data)$value$y1)
X <- source(data)$value$X
k <- dim(X)[2]
X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")

# Marg Likelihood
# ------------------------------------------------------------       
           
nsimu<-length(theta[,1])*10

# generation
thetamean<-apply(theta,2,mean)
thetavar<-cov(theta)
thetasimu<-rmnorm(nsimu,thetamean,thetavar)

# importance
c <- c*b
logjointe <- LogJointLogit_Zellner(X,thetasimu,theta0,a,c,y1,ytot)

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T) 
logpred <- logjointe-logimportance
K <- mean(logpred)

estimlogypred_M0 <- K + log(mean(exp(logpred-K)))

weight <- exp(logpred-K)
weight <- weight/sum(weight)
# plot((1:nsimu)/nsimu, cumsum(sort(weight)))
# lines((1:nsimu)/nsimu,(1:nsimu)/nsimu)



# ---------------------------------------------------------
model.version <- "M1"
# ---------------------------------------------------------

data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
n <- length(source(data)$value$y1)
X <- source(data)$value$X
k <- dim(X)[2]
X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")

# Marg Likelihood
# ------------------------------------------------------------      
           
nsimu<-length(theta[,1])*10

# generation
thetamean<-apply(theta,2,mean)
thetavar<-cov(theta)
thetasimu<-rmnorm(nsimu,thetamean,thetavar)

# importance
c <- c*b
logjointe <- LogJointLogit_Zellner(X,thetasimu,theta0,a,c,y1,ytot)

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T) 
logpred <- logjointe-logimportance
K <- mean(logpred)

estimlogypred_M1 <- K + log(mean(exp(logpred-K)))

weight <- exp(logpred-K)
weight <- weight/sum(weight)
# plot((1:nsimu)/nsimu, cumsum(sort(weight)))
# lines((1:nsimu)/nsimu,(1:nsimu)/nsimu)



# ---------------------------------------------------------
model.version <- "M2"
# ---------------------------------------------------------


data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
n <- length(source(data)$value$y1)
X <- source(data)$value$X
k <- dim(X)[2]
X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")

# Marg Likelihood
# ------------------------------------------------------------      
           
nsimu<-length(theta[,1])*10

# generation
thetamean<-apply(theta,2,mean)
thetavar<-cov(theta)
thetasimu<-rmnorm(nsimu,thetamean,thetavar)

# importance
c <- c*b
logjointe <- LogJointLogit_Zellner(X,thetasimu,theta0,a,c,y1,ytot)

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T) 
logpred <- logjointe-logimportance
K <- mean(logpred)

estimlogypred_M2 <- K + log(mean(exp(logpred-K)))

weight <- exp(logpred-K)
weight <- weight/sum(weight)
# plot((1:nsimu)/nsimu, cumsum(sort(weight)))
# lines((1:nsimu)/nsimu,(1:nsimu)/nsimu)



# ---------------------------------------------------------
model.version <- "M3"
# ---------------------------------------------------------

data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
n <- length(source(data)$value$y1)
X <- source(data)$value$X
k <- dim(X)[2]
X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")

# Marg Likelihood
# ------------------------------------------------------------    
           
nsimu <- length(theta[,1])*10

# generation
thetamean <- apply(theta,2,mean)
thetavar <- cov(theta)
thetasimu <- rmnorm(nsimu,thetamean,thetavar)

# importance
c <- c*b
logjointe <- LogJointLogit_Zellner(X,thetasimu,theta0,a,c,y1,ytot)

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T) 
logpred <- logjointe-logimportance
K <- mean(logpred)

estimlogypred_M3 <- K + log(mean(exp(logpred-K)))

weight <- exp(logpred-K)
weight <- weight/sum(weight)
# plot((1:nsimu)/nsimu, cumsum(sort(weight)))
# lines((1:nsimu)/nsimu,(1:nsimu)/nsimu)



# ---------------------------------------------------------
model.version <- "M4"
# ---------------------------------------------------------

data <- paste("data_GLMpSm1_Zellner_",model.version,".txt", sep = "")
n <- length(source(data)$value$y1)
X <- source(data)$value$X
k <- dim(X)[2]
X <- t(matrix(X, k, n))
y1 <- source(data)$value$y1
ytot <- source(data)$value$ytot
theta0 <- source(data)$value$theta0
a <- source(data)$value$a
c <- source(data)$value$c
b <- source(data)$value$b

file_to_read <- paste("theta_",model.version,".txt",sep = "")
theta <- read.table(file = file_to_read, header = T, sep = "")

# Marg Likelihood
# ------------------------------------------------------------      
           
nsimu<-length(theta[,1])*10

# generation
thetamean <- apply(theta,2,mean)
thetavar <- cov(theta)
thetasimu <- rmnorm(nsimu,thetamean,thetavar)

# importance
c <- c*b
logjointe <- LogJointLogit_Zellner(X,thetasimu,theta0,a,c,y1,ytot)

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T) 
logpred <- logjointe-logimportance
K <- mean(logpred)

estimlogypred_M4 <- K + log(mean(exp(logpred-K)))

weight <- exp(logpred-K)
weight <- weight/sum(weight)




# windows()

res <- 6
name_figure <- "ISPredictiveM4.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

size.text <- 1.3

plot(	(1:nsimu)/nsimu, cumsum(sort(weight)),typ='l', 
		ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
lines((1:nsimu)/nsimu,(1:nsimu)/nsimu, lty=2)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Importance weights", bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Cumulative distribution", bty="n", line=3, cex=size.text, font = 1)

dev.off()



# -------------------------------------------------------
# Log marg likelihood  and  Bayes factor
# -------------------------------------------------------

log_predictive <- c(estimlogypred_M0,estimlogypred_M1,estimlogypred_M2,estimlogypred_M3,estimlogypred_M4)
print(log_predictive)

BF <- matrix(NA,5,5)
for (i in 1:5)
{
BF[i,] <- exp(log_predictive[i]-log_predictive)
}
              
print(BF)




